/* Demonstration Schieberegister mit 5x7 Matrix
 * 
 * PORTB: Zeilensteuerung
 * PC0: Takt
 * PC1: Data
 */

#include <avr/io.h>
#include <util/delay.h>		// _delay_ms() 

void delay_ms(uint16_t);
void set_row (uint8_t);
void write_rows (void);
int main (void);

#define DISPCOLS 15					// Anzahl der Spalten 
volatile uint8_t memory[DISPCOLS];	// Speicher fr Display, spaltenweise orientiert, Index 0=links


// wartet ms Millisekunden
void delay_ms(uint16_t ms)
{
	for(uint16_t t=0; t<=ms; t++)
		_delay_ms(1); 
}

/**
	@brief	Schiebt die Daten fr die Zeile in die Schieberegister.
			Daten werden aus dem globalen Array memory genommen
	@param	Zeile (0..6)
	@return	none
*/
void set_row (uint8_t row)
{
	int8_t i;
	
	// Alle Spalten des Displays durchlaufen. Mit rechter Spalte beginnen
	for (i = DISPCOLS-1; i>=0; i--)		
	{
		if ((memory[i] >> (6-row)) & 1)	// Bit in der Spalte an der bergebenden Zeile gesetzt?
			PORTC |= (1 << PC1);		// Data Bit setzen
		else
			PORTC &= ~(1 << PC1);		// Data Bit lschen
		
		PORTC |= (1 << PC0);	// Takt HIGH -> Flanke => schieben
		PORTC &= ~(1 << PC0);	// Takt LOW 
	}
}

/**
	@brief	Schaltet die Zeilen nacheinander durch und sorgt so fr die Ausgabe
			auf dem Display. Mu endlos aufgerufen werden.
	@param	none
	@return	none
*/
void write_rows ()
{
	uint8_t i;

	for (i=0; i<=6; i++)	// Alle Zeilen durchlaufen
	{
		PORTB = 0;			// alle Zeilen aus
		set_row (i);		// Daten fr aktuelle Zeile in Schieberegister schieben
		PORTB = (1 << i);	// aktuelle Zeile an
		delay_ms (2);		// Zeile leuchten lassen
	}
}

int main ()
{
	DDRC  |= (1 << DDC0) | (1 << DDC1);		// PC0, PC1 als Ausgang
	PORTC &= ~((1 << PC0) | (1 << PC1));	// PC0, PC1 aus
	DDRB  = 0xFF; 							// PortB als Ausgang
	PORTB = 0;								// PortB aus
	
	// beliebiges Muster spaltenweise definieren. 
	// HSB unwichtig. LSB = unterste Zeile
	memory[0]  = 0b01111110;
	memory[1]  = 0b00101010;
	memory[2]  = 0b01010101;
	memory[3]  = 0b00101010;
	memory[4]  = 0b01010101;
	memory[5]  = 0b00101010;
	memory[6]  = 0b01010101;
	memory[7]  = 0b00101010;
	memory[8]  = 0b01010101;
	memory[9]  = 0b00101010;
	memory[10] = 0b01010101;
	memory[11] = 0b00101010;
	memory[12] = 0b01010101;
	memory[13] = 0b00101010;
	memory[14] = 0b00111111;
	
	while (1)		// Endlos Zeilen ausgeben
		write_rows ();

	return 0;
}
