/* Ansteuerung von drei 7-Segmentanzeige im Multiplexbetrieb
 * Gemeinsame Anode, PB0..PB6
 * Spalten: PC0..PC2
 *
 *    a
 *  f   b
 *    g
 *  e   c
 *    d
 *
 */

#include <avr/io.h>
#include <util/delay.h>		// definiert _delay_ms() 

void delay_ms(uint16_t);
void digit (uint8_t, uint8_t);
void out_zahl (uint16_t);
int main (void);

const int8_t numbers [10] =
{
//    1gfedcba
	0b11000000,		// 0
	0b11111001,		// 1
	0b10100100,		// 2
	0b10110000,		// 3
	0b10011001,		// 4
	0b10010010,		// 5
	0b10000010,		// 6
	0b11111000,		// 7
	0b10000000,		// 8
	0b10010000,		// 9
};

// wartet ms Millisekunden
void delay_ms (uint16_t ms)
{
  for(uint16_t t=0; t<=ms; t++)
	_delay_ms(1); 
}

/**
	@brief	Gibt die Zahl an der Position aus
	@param 	auszugebende Zahl
			Pinbezeichner fr die Stelle/Digit
	@return	none
*/
void digit (uint8_t wert, uint8_t pin)
{
	PORTC |= (1 << PC0) | (1 << PC1) | (1 << PC2);		// 0..2 High = Digits aus
	PORTB = numbers[wert];								// Zahl ausgeben
	PORTC &= ~(1 << pin);								// Low = entsprechendes Digit an
	delay_ms (5);
}

/**
	@brief	gibt eine int-Zahl auf dem Display aus. 
	@param 	Zahl (0..999)
	@return	none
*/
void out_zahl (uint16_t zahl)       
{
	digit (zahl/100, PC0);	// Hunderter Dezimalstelle 
	zahl = zahl % 100;
		
	digit (zahl/10, PC1);	// Zehner Dezimalstelle 
	zahl = zahl % 10;
	
	digit (zahl, PC2);		// Einer Dezimalstelle 
}

int main()
{
	DDRB = 0xFF;										// PortB alles als Ausgang
	DDRC |= (1 << DDC0) | (1 << DDC1) | (1 << DDC2);	// PortC 0..2 als Ausgang
	PORTC |= (1 << PC0) | (1 << PC1) | (1 << PC2);		// 0..2 High = Digits aus

	while (1)	// Endlos
	{
		out_zahl(456);
	}
	return 0;
}


