/* Ansteuerung von drei 7-Segmentanzeige im Multiplexbetrieb
 * Gemeinsame Anode, PB0..PB6
 * Spalten: PC0..PC2
 *
 *    a
 *  f   b
 *    g
 *  e   c
 *    d
 *
 */

#include <avr/io.h>
#include <util/delay.h>		// definiert _delay_ms() 
#include <avr/interrupt.h>

void delay_ms(uint16_t);
void digit (uint8_t, uint8_t);
void out_zahl (uint16_t);
int main (void);

const int8_t numbers [10] =
{
//    1gfedcba
	0b11000000,		// 0
	0b11111001,		// 1
	0b10100100,		// 2
	0b10110000,		// 3
	0b10011001,		// 4
	0b10010010,		// 5
	0b10000010,		// 6
	0b11111000,		// 7
	0b10000000,		// 8
	0b10010000,		// 9
};
volatile uint8_t hunderter, zehner, einer;	// Speicher fr die einzelnen Digits
volatile uint8_t position = 0;				// Segment, welches gerade dran ist

/**
	@brief	Wird vom 16 Bit Timer ausgelst, wenn dieser den Vergleichwert erreicht
	@param	none
	@return	none
*/
SIGNAL (SIG_OUTPUT_COMPARE1A)
{
	PORTC |= (1 << PC0) | (1 << PC1) | (1 << PC2);		// 0..2 High = Digits aus

	switch (position)		// Aktuelle Stelle ausgeben
	{
		case 0 : digit (hunderter, PC0); break;
		case 1 : digit (zehner, PC1); break;
		case 2 : digit (einer, PC2); break;
	}

	position ++;			// beim nchsten Mal nchstes Digit
	if (position == 3) 		// wenn alle Digits durch 
		position = 0;		// von vorne beginnen
}

// wartet ms Millisekunden
void delay_ms (uint16_t ms)
{
	for(uint16_t t=0; t<=ms; t++)
		_delay_ms(1); 
}

/**
	@brief	Gibt die Zahl an der Position aus
	@param 	auszugebende Zahl
			Pinbezeichner fr die Stelle/Digit
	@return	none
*/
void digit (uint8_t wert, uint8_t pin)
{
	PORTB = numbers[wert];								// Zahl ausgeben
	PORTC &= ~(1 << pin);								// Low = entsprechendes Digit an
}

/**
	@brief	Zerlegt eine Zahl in drei Stellen und bereitet sie so fr die Ausgabe vor
	@param 	Zahl (0..999)
	@return	none
*/
void zahl_ausgeben (uint16_t zahl)       
{
	hunderter = zahl/100;	// Hunderter Dezimalstelle 
	zahl = zahl % 100;
		
	zehner = zahl/10;		// Zehner Dezimalstelle 
	zahl = zahl % 10;
	
	einer = zahl;			// Einer Dezimalstelle 
}

int main()
{
	DDRB = 0xFF;										// PortB alles als Ausgang
	DDRC |= (1 << DDC0) | (1 << DDC1) | (1 << DDC2);	// PortC 0..2 als Ausgang
	PORTC |= (1 << PC0) | (1 << PC1) | (1 << PC2);		// 0..2 High = Digits aus

	OCR1A = (uint16_t)(F_CPU/1024.0*0.005);			// Vergleichwert fr 0,005s laden: Takt= 8MHz/1024 = 7812,5 Zhlerschritte pro s
	TIMSK |= (1<<OCIE1A); 							// Interrupt wenn Timer Vergleichswert erreicht
	TCCR1B = _BV(CS12) | _BV(CS10) | _BV(WGM12);	// Timer mit Div 1024 starten und Clear Counter on Compare Match
	sei();											// IRQ Behandlung ein

	while (1)	// Endlos
	{
		for (uint16_t i = 0; i <= 999; i++)
		{
			zahl_ausgeben (i);
			delay_ms (100);
		}
	}
	return 0;
}


