/* DCF77
 *
 *
 */

#include "lcd_tools.h"
#include <avr/signal.h>		// Timer
#include <avr/interrupt.h>	// IRQ

#define DCF_DD		DDB1
#define DCF_DDR		DDRB
#define DCF_PIN		PINB
#define DCF_PINPin	PINB1

#define LED_DD		DDB0
#define LED_DDR		DDRB
#define LED_Port	PORTB
#define LED_Pin		PB0

volatile uint16_t	timer0_cnt = 0;
volatile uint8_t 	sync,
					sekunde = 0, 
					minute = 0,
					stunde = 0,
					wo_tag = 0,
					tag = 0,
					monat = 0,
					jahr = 0,
					mesz = 0,
					n_minute = 0,
					n_stunde = 0,
					n_wo_tag = 0,
					n_tag = 0,
					n_monat = 0,
					n_jahr = 0,
					n_mesz = 0;
					
#define sek_teil F_CPU/1/256		// Takt/Prescaler = Timerimpulse pro Sekunde. 
									// Alle 256 Impulse ein berlauf. (=31.250 berlufe entsprechen 1 Sekunde)

/**
	@brief	Wird vom 8-Bit Timer0 bei berlauf ausgelst. Alle 256 CPU Takte einmal. 
			In sek_teil steht, wieviele berlufe 1 Sekunde entsprechen
	@param	none
	@return	none
*/
ISR (TIMER0_OVF_vect)
{
	timer0_cnt++;
}

/**
	@brief	
	@param	none
	@return	none
*/
void zeige_status (uint8_t status)
{
	lcd_gotopos (1,20);
	lcd_writechar (status);		
}

/**
	@brief	Gibt die Zeit aus & lt die Sekunden weiterlaufen
	@param	none
	@return	none
*/
void zeige_uhr ()
{
	if (timer0_cnt >= sek_teil)		// wenn Timer intern 1 Sekunde gemessen hat, 
	{								// dann nchste Sekunde und ggf. bei 60. Sekunde berlauf -> Minute -> Stunde 
		timer0_cnt = 0;

		if (sync == 0)
		{
			sekunde++;
			if (sekunde == 60)
			{
				sekunde = 0;
				minute++;
			}
			if (minute == 60)
			{
				minute = 0;
				stunde++;
			}
			if (stunde == 24)
				stunde = 0;
		}
	}		
	lcd_gotopos (1,10);
	lcd_writezahl (stunde, 2, 0);
	lcd_writetext (":");
	lcd_writezahl (minute, 2, 0);
	lcd_writetext (":");
	lcd_writezahl (sekunde, 2, 0);
	if (mesz)		// Sommerzeit?
		lcd_writetext (" s");
}

/**
	@brief	Gibt das Datum aus
	@param	none
	@return	none
*/
void zeige_datum ()
{
	lcd_gotopos (2,10);
	lcd_writezahl (tag, 2, 0);
	lcd_writetext (".");
	lcd_writezahl (monat, 2, 0);
	lcd_writetext (".");
	lcd_writezahl (jahr, 2, 0);

	switch (wo_tag)
	{
		case 1 : lcd_writetext (" Mo"); break;
		case 2 : lcd_writetext (" Di"); break;
		case 3 : lcd_writetext (" Mi"); break;
		case 4 : lcd_writetext (" Do"); break;
		case 5 : lcd_writetext (" Fr"); break;
		case 6 : lcd_writetext (" Sa"); break;
		case 7 : lcd_writetext (" So"); break;
	}
}

/**
	@brief	Wartet darauf, den Sync-Impuls via DCF77 zu empfangen. 
	@param	none
	@return	none
*/
void warte_bis_sync ()
{
	uint16_t 	ticks,
				msecs = 0;
	uint8_t 	timer_cntl,
				timer_cnth;
	
	do
	{
		msecs = 0;
		zeige_uhr();
		if (DCF_PIN & (1 << DCF_PINPin))	// HIGH
		{
			TCNT1H = 0; 	
			TCNT1L = 0; 	
			TCCR1B = (1<<CS00)|(1<<CS02);	// Timer mit Div 1024 starten
		
			while (DCF_PIN & (1 << DCF_PINPin))
			{
				zeige_uhr();	// solange HIGH: warte und zeige Zeit
			}
			
			TCCR1B = 0;		// Stop
			timer_cntl = TCNT1L;
			timer_cnth = TCNT1H;
			
			ticks = timer_cnth * 0x100 + timer_cntl;
			msecs = ticks / (F_CPU/1024.0/1000.0);	// Takt= 8MHz => 8.000.000/1024 = 7812,5 => 7,8125 Zhlerschritte pro ms
		}
	}
	while (msecs < 1700);	// mindestens 1700ms HIGH (59. Bit=LOW + Rest vom 58. Bit) => Sync erkannt
}


int main()
{
	uint16_t 	ticks,
				msecs = 0;
	uint8_t 	bit_cnt,
				frame[60],
				sync,
				timer_cntl,
				timer_cnth;

	LED_DDR = (1 << LED_DD);			// Pxn als Ausgang (LED)
	DCF_DDR &= ~(1 << DCF_DD);			// Pxn als Eingang (DCF77)

	lcd_ini();
	// eigenes Zeichen definieren
	lcd_write (0x40, 0);		// Zeichen #0 schreiben Antenne/Empfang  OK
	lcd_write (0b00001, 1);		//     #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00100, 1);		//   #
	lcd_write (0b01010, 1);		//  # #
	lcd_write (0b10010, 1);		// #  #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #

	lcd_write (0x48, 0);		// Zeichen #1 schreiben kein Empfang
	lcd_write (0b00000, 1);		//      
	lcd_write (0b00000, 1);		//     
	lcd_write (0b00100, 1);		//   #
	lcd_write (0b00010, 1);		//    #	
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #

	lcd_cls();				// Display lschen (nach Zeichendefinition erforderlich!)
	
	// 8-Bit Timer0 fr Sekunden-Takt
	TCNT0 = 0;					// Startwert
	TIMSK |= (1 << TOIE0);		// IRQ bei berlauf 0xFF -> 0x00
	TIFR  |= (1 << TOV0); 		// Clear TOV0 clear pending interrupts
	TCCR0 |= (1 << CS00);		// Timer mit CPU/1 Prescaler, da so bei 8 MHz kein Rest bei Division bleibt
	sei();						// Interrupts zulassen

	zeige_status (1);
	warte_bis_sync();
	zeige_status (0);

	bit_cnt = 0;

	sync = 0;
	while(1)
	{
		if (!(DCF_PIN & (1 << DCF_PINPin)))	// LOW
		{
			TCNT1H = 0; 	// Takt= 8MHz => 8.000.000/1024 = 7812,5 => 7,8125 Zhlerschritte pro ms
			TCNT1L = 0; 	// Vergleichswert laden 3 Sekunden
			TCCR1B = (1<<CS00)|(1<<CS02);						// Timer mit Div 1024 starten

			LED_Port |= (1 << LED_Pin);	// LED ein
			
			while (!(DCF_PIN & (1 << DCF_PINPin)))	// solange LOW: warte und zeige ggf. neue Uhrzeit an
			{
				if (sync && (bit_cnt == 0))		// wenn neue Zeit empfangen wurde und 1. Bit nach Syncpause (= Start)
				{
					timer0_cnt = 0;		// interner Timer auf Anfang
					
					sekunde = 0; 		// empfangene Uhrzeit fr Anzeige bernehmen
					minute = n_minute;
					stunde = n_stunde;
					wo_tag = n_wo_tag;
					tag = n_tag;
					monat = n_monat;
					jahr = n_jahr;
					mesz = n_mesz;
					
					zeige_uhr();
					zeige_datum();
					
					sync = 0;
				}
			}

			LED_Port &= ~(1 << LED_Pin);	// LED aus
			
			TCCR1B = 0;		// Stop
			timer_cntl = TCNT1L;
			timer_cnth = TCNT1H;
			
			ticks = timer_cnth * 0x100 + timer_cntl;
			msecs = ticks / (F_CPU/1024.0/1000.0);	// Takt= 8MHz => 8.000.000/1024 = 7812,5 => 7,8125 Zhlerschritte pro ms

			if (msecs < 130)			// kurze LOW Phase = 0
			{
				frame[bit_cnt] = 0;
			}
			if (msecs > 170)			// lange LOW Phase = 1
			{
				frame[bit_cnt] = 1;
			}
			
			bit_cnt++;
		}

		if (bit_cnt == 59)
		{
			// XOR Verknpfung fr Fehlerprfung:
			if ((frame[21] ^ frame[22] ^ frame[23] ^ frame[24] ^ frame[25] ^ frame[26] ^ frame[27] ^ frame[28]) ||
			    (frame[29] ^ frame[30] ^ frame[31] ^ frame[32] ^ frame[33] ^ frame[34] ^ frame[35]) ||
				(frame[36] ^ frame[37] ^ frame[38] ^ frame[39] ^ frame[40] ^ frame[41] ^ frame[42] ^ frame[43] ^ frame[44] ^ frame[45] ^ frame[46] ^ frame[47] ^ frame[48] ^ frame[49] ^ frame[50] ^ frame[51] ^ frame[52] ^ frame[53] ^ frame[54] ^ frame[55] ^ frame[56] ^ frame[57] ^ frame[58]) ||
				!frame[20])
			{
				// Empfangene Zeit fehlerhaft
				zeige_status (1);
				bit_cnt++;
			}
			else
			{	// Empfangene Zeit gltig => Interne Zeit setzen
				if (frame[17])	n_mesz = 1;
				if (frame[18])	n_mesz = 0;
				
				n_minute = 0;
				if (frame[21])	n_minute += 1;
				if (frame[22])	n_minute += 2;
				if (frame[23])	n_minute += 4;
				if (frame[24])	n_minute += 8;
				if (frame[25])	n_minute += 10;
				if (frame[26])	n_minute += 20;
				if (frame[27])	n_minute += 40;
	
				n_stunde = 0;
				if (frame[29])	n_stunde += 1;
				if (frame[30])	n_stunde += 2;
				if (frame[31])	n_stunde += 4;
				if (frame[32])	n_stunde += 8;
				if (frame[33])	n_stunde += 10;
				if (frame[34])	n_stunde += 20;
	
				n_tag = 0;
				if (frame[36])	n_tag += 1;
				if (frame[37])	n_tag += 2;
				if (frame[38])	n_tag += 4;
				if (frame[39])	n_tag += 8;
				if (frame[40])	n_tag += 10;
				if (frame[41])	n_tag += 20;
	
				n_wo_tag = 0;
				if (frame[42])	n_wo_tag += 1;
				if (frame[43])	n_wo_tag += 2;
				if (frame[44])	n_wo_tag += 4;
	
				n_monat = 0;
				if (frame[45])	n_monat += 1;
				if (frame[46])	n_monat += 2;
				if (frame[47])	n_monat += 4;
				if (frame[48])	n_monat += 8;
				if (frame[49])	n_monat += 10;
	
				n_jahr = 0;
				if (frame[50])	n_jahr += 1;
				if (frame[51])	n_jahr += 2;
				if (frame[52])	n_jahr += 4;
				if (frame[53])	n_jahr += 8;
				if (frame[54])	n_jahr += 10;
				if (frame[55])	n_jahr += 20;
				if (frame[56])	n_jahr += 40;
				if (frame[57])	n_jahr += 80;
					
				zeige_status (0);

				bit_cnt = 0;
				sync = 1;	
			}
		}

		zeige_uhr();
		
		if (bit_cnt > 59)		// Sync verloren => Neustart
		{
			bit_cnt = 0;
			warte_bis_sync();
		}
	}
}


