/* Empfngt GPS Daten am USART und stellt diese auf dem LCD dar
 *
 * 4800Bd, 8N1
 *
 */

#include <string.h>
#include <stdlib.h>
#include <avr/io.h>
#include "lcd_tools.h"
#include "uart.h"

void getTeilstring (char *, char *, uint8_t);
int main(void);

/**
	@brief	Such im Quellstring nach dem n-ten vorkommen des Zeichen ',' und kopiert den String ab 
			dem darauffolgenden Zeichen bis zum nchsten Zeichen ',' (ohne dieses).
			Liefert leeren String, wenn zwischen beiden ',' kein Zeichen steht.
	@param	Zielstring
			Quellstring
			ab dem wievielten ',' kopieren?
	@return	none
*/
void getTeilstring (char *dest, char *from, uint8_t pos)
{
	uint8_t	detect=0,			// gesuchtes n-tes vorkommen des Zeichen gefunden = 1
			cnt=0;
	
	while (*from != '\0')		// Zeichenkette durchlaufen bis Ende
	{
		if (*from == ',')		// wenn gesuchtes Zeichen gefunden wurde
			cnt++;				// das wievielte mal wurde das gesuchte Zeichen gefunden?
		if (cnt == pos)			// wenn das gesuchte Zeichen zum n-ten mal gefunden wurde: kopiere String bis zum nchsten auftreten
		{
			if (detect == 0)	// Bisher kein Flag gesetzt
				detect = 1;		// Flag dafr, da das Zeichen zum n-ten mal gefunden wurde setzen
			else				// nur wenn das Flag gesetzt wurde, dann die Zeichen kopieren. 
			{					// So wird das gesuchte Zeichen selbt nicht mitkopiert
				*dest = *from;	// Zeichen kopieren
				dest++;			// nchstes zeichen im Zielstring whlen
			}
		}
		from++;					// nchstes Zeichen im Quellstring whlen
	}
	*dest = '\0';				// Zielstring mit Endmarkierung versehen
}


int main(void)
{
	char zeichen;
	char message[83];	// maximal 82 Zeichen pro Datensatz
	char tmp[83];		// maximal 82 Zeichen pro Datensatz
	char tmp2[83];		// maximal 82 Zeichen pro Datensatz
	char *string;
	char *vorkomma, *nachkomma;
	uint8_t i;
	uint16_t zahl;
	double komma;
	
	DDRB = (1 << DDB0);		// PB0 Ausgang (LED);
	uart_ini();
	lcd_ini();
	// eigenes Zeichen definieren
	lcd_write (0x40, 0);		// Zeichen #0 schreiben Antenne/Empfang  OK
	lcd_write (0b00001, 1);		//     #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00100, 1);		//   #
	lcd_write (0b01010, 1);		//  # #
	lcd_write (0b10010, 1);		// #  #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #

	lcd_write (0x48, 0);		// Zeichen #1 schreiben kein Empfang
	lcd_write (0b00000, 1);		//      
	lcd_write (0b00000, 1);		//     
	lcd_write (0b00100, 1);		//   #
	lcd_write (0b00010, 1);		//    #	
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #
	lcd_write (0b00010, 1);		//    #

	lcd_cls();				// Display lschen (nach Zeichendefinition erforderlich!)
	
//	lcd_writetext (">");		// Auf LCD
	
	while (1)
	{
		i = 0;
		do	// Kompletten Datensatz empfangen
		{
			zeichen = ser_getc();		// Zeichen vom USAT einlesen
			message[i] = zeichen;		// Zeichenkette erweitern
			i++;
		} while ( zeichen != '\n');		// Zeilende v. Datensatz erkennen (CR+LF)

		message[i-2] = '\0';	// vorletztes Zeichen (CR) durch Stringendekennung ersetzen

		if (strncmp  (message, "$GPRMC", 6) == 0)		// GPRMC Datensatz empfangen
		{
			getTeilstring (tmp, message, 2);		// = Status A/V
			lcd_gotopos (1,20);
			if (tmp[0] == 'V')
			{
				lcd_writechar (1);				// Daten nicht OK Zeichen
				PORTB &= ~(1<<PB0);				// LED an PB0 aus
			}
			else
			{
				lcd_writechar (0);				// Daten OK Zeichen
				PORTB |= (1<<PB0);				// LED an PB0 ein
			}

			getTeilstring (tmp, message, 1);	// = Uhrzeit UTC
			lcd_gotopos (1,1);
			lcd_writetext ("Zeit: ");
			lcd_writechar (tmp[0]);				// h
			lcd_writechar (tmp[1]);				// h
			lcd_writechar (':');
			lcd_writechar (tmp[2]);				// m
			lcd_writechar (tmp[3]);				// m
			lcd_writechar (':');
			lcd_writechar (tmp[4]);				// s
			lcd_writechar (tmp[5]);				// s

			getTeilstring (tmp, message, 3);	// = Latitude (z. B. 5256.0361)
			lcd_gotopos (2,1);
			lcd_writetext ("Lat.:   ");
			string = tmp;
			vorkomma = strsep (&string, ".");
			nachkomma = strsep (&string, ".");
			zahl = atoi (vorkomma);
			zahl = (uint16_t) zahl/100;			// Vorkommaanteil separieren (5256 => 52,56 => 52)
			lcd_writezahl (zahl, 2, 1);			// Grad
			lcd_writechar ('');				// ""
			strrev (tmp);						// String rckwrts (01248 => 84210)
			strncpy (tmp2, tmp, 2);				// die vormals letzten zwei jetzt ersten zwei) Zeichen kopieren (= 84)
			tmp2[2] = '\0';						// Endemarkierung anhngen notwendig
			strrev (tmp2);						// String nochmal rckwrts = wieder vorwrts (= 48)
			lcd_writetext (tmp2);				// Minuten
			lcd_writechar ('\'');				// "'"
			komma = atof (tmp);					// komplette Zahl
			zahl = atoi (nachkomma);			
			komma = zahl*0.006;					// Sekunden berechnen (0361*0,006 = 2,166)
			dtostrf  (komma, 3, 2, tmp);		// Zwei Nachkommastellen zeigen
			lcd_writetext (tmp);				// Sekunden
			lcd_writechar ('"');				// """

			getTeilstring (tmp, message, 4);	// = Hemisphre
			lcd_gotopos (2,7);
			lcd_writetext (tmp);

			getTeilstring (tmp, message, 5);	// = Longitude (z. B. 01248.0766)
			lcd_gotopos (3,1);
			lcd_writetext ("Lng.:  ");
			string = tmp;
			vorkomma = strsep (&string, ".");
			nachkomma = strsep (&string, ".");
			zahl = atoi (vorkomma);
			zahl = (uint16_t) zahl/100;			// Vorkommaanteil separieren 
			lcd_writezahl (zahl, 3, 1);			// Grad
			lcd_writechar ('');				// ""
			strrev (tmp);						// String rckwrts (01248 => 84210)
			strncpy (tmp2, tmp, 2);				// die vormals letzten zwei jetzt ersten zwei) Zeichen kopieren (= 84)
			tmp2[2] = '\0';						// Endemarkierung anhngen notwendig
			strrev (tmp2);						// String nochmal rckwrts = wieder vorwrts (= 48)
			lcd_writetext (tmp2);				// Minuten
			lcd_writechar ('\'');				// "'"
			komma = atof (tmp);					// komplette Zahl
			zahl = atoi (nachkomma);			
			komma = zahl*0.006;					// Sekunden berechnen 
			dtostrf  (komma, 3, 2, tmp);		// Zwei Nachkommastellen zeigen
			lcd_writetext (tmp);				// Sekunden
			lcd_writechar ('"');				// """
			
			getTeilstring (tmp, message, 6);	// = Hemisphre
			lcd_gotopos (3,7);
			lcd_writetext (tmp);
		}	// GPRMC
		if (strncmp  (message, "$GPGGA", 6) == 0)		// GPGGA Datensatz empfangen
		{
			getTeilstring (tmp, message, 7);		// Sat count
			lcd_gotopos (4,1);
			lcd_writetext ("Sat: ");
			lcd_writetext (tmp);
		
			getTeilstring (tmp, message, 9);		// Antenna height above/below mean sea level
			lcd_gotopos (4,9);
			lcd_writetext ("Alt: ");
			lcd_writetext (tmp);
		
		}	// GPGGA 
	}
	return 0;
} 

