/* Software PWM mit 8-Bit Timer 0
 *
 * Dimmt eine LED an PB0 . Dateneingabe ueber RS232
 * Prescaler: 1/256
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#define UART_BAUD_RATE 9600L
#define UART_BAUD_CALC(UART_BAUD_RATE,F_CPU) ((F_CPU)/((UART_BAUD_RATE)*16L)-1)

// USART, Rx Complete
ISR (USART_RXC_vect) 		
{
	OCR0 = UDR; 	// Byte abholen, sonst Endlosinterrupt
					// und als Vergleichswert nutzen
	TCNT0 = 0xFF;	// MAX in Timerzaehler => gleich Ueberlauf
}

// Timer/Counter0 Overflow 
ISR (TIMER0_OVF_vect)
{
	PORTB |= (1 << PB0);		// PB0 ein
}

// Timer/Counter0 Compare Match
ISR (TIMER0_COMP_vect)
{
	PORTB &= ~(1 << PB0);		// PB0 aus
}

int main()
{
	DDRB = (1 << DDB0);			// PORTB als Ausgang
	
	UCSRB |= (1 << RXEN );											// UART RX (empfangen) einschalten
	UCSRB |= (1 << RXCIE);											// Interruptauslsung fr eingehende Daten aktivieren
	UCSRC |= (1<<URSEL)|(1<<UCSZ0)|(1<<UCSZ1);						// Asynchron, 8N1
	UBRRH=(uint8_t)(UART_BAUD_CALC(UART_BAUD_RATE,F_CPU)>>8);		// Baudrate whlen
	UBRRL=(uint8_t)UART_BAUD_CALC(UART_BAUD_RATE,F_CPU);

	TIMSK |= (1 << TOIE0) | (1 << OCIE0);	// IRQ bei berlauf + Compare Match
	OCR0 = 0x80;							// Vergleichswert 128 einstellen
	sei();									// IRQs enable
	PORTB |= (1 << PB0);					// PB0 ein
	TCCR0 |= (1 << CS02);		// Prescaler: 1/256 

	while (1);		// endlos
	return 0;
}
