/* Serial Peripheral Interface
 *
 * Slave
 *
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#define DDR_SPI		DDRB	// SPI Portrichtungsregister
#define DD_MISO		DDB4	// SPI Pin MISO

void SPI_SlaveInit(void);
char SPI_SlaveReceive(void);
void SPI_SlaveTransmit(char);
int main (void);

/**
	@brief	Interruptbehandlung bei Empfang abgeschlossen
	@param	Byte
	@return	none
*/
ISR (SPI_STC_vect)
{
	char zahl;
	zahl = SPDR;					// Zeichen aus Empfangspuffer holen
	if (zahl == 1)
		SPI_SlaveTransmit(100);
	else
		SPI_SlaveTransmit(0);
		
	DDRD |= (1 << DDD0);			// PD0 als Ausgang fuer LED
	PORTD |= (1 << PD0);			// LED ein
}

/**
	@brief	Slave Initialisierung: Portrichtung und Uebertragungsparameter setzen
	@param	Byte
	@return	none
*/
void SPI_SlaveInit(void)
{
	DDR_SPI = (1<<DD_MISO);			// MISO Ausgang, alle andere Eingang
	SPCR = (1<<SPE)|(1<<SPIE);		// Enable SPI
}

/**
	@brief	Byte senden
	@param	Byte
	@return	none
*/
void SPI_SlaveTransmit(char data)
{
	SPDR = data;					// Uebertragung starten
	while(!(SPSR & (1<<SPIF)));		// warte bis Uebertragung abgeschlossen
}


int main(void)
{
	sei();				// Interruptbehandlung ein
	SPI_SlaveInit();
	while(1);			// endlos 
	
	return 0;
}



